static char *versionRCS =
"$Header$";
// *created  "Thu Aug  7 13:00:20 2003" *by "Paul E. Black"
// *modified "Fri Aug  8 10:18:08 2003" *by "Paul E. Black"

//-----------------------------------------------------------------------------
//
// This software was developed at the National Institute of Standards 
// and Technology by employees of the Federal Government in the course 
// of their official duties.  Pursuant to title 17 Section 105 of the 
// United States Code this software is not subject to copyright 
// protection and is in the public domain. 
// 
// We would appreciate acknowledgment if the software is used.
//
// Paul E. Black  paul.black@nist.gov
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	Test the utilities for reasonableness
//-----------------------------------------------------------------------------

#include <stdlib.h>

// GFV files
#include "infolines.h"
#include "timer.h"

static char *useMsg = "selftest [--version]";
static char *progVersion = "1.0";

// this explains Mebi- prefix, which is 2^20
static char *binaryUnitsURL = "http://physics.nist.gov/cuu/Units/binary.html";

static void memoryTest()
{
    // get base memory
    // Note: getMem() returns memory in Mebibytes or 2^20 bytes
    MemUsage mu;
    double baseMem_Mib = mu.getMem();

    //----------------------------------------------------------------
    // make sure measured memory increases approximately as expected
    //----------------------------------------------------------------
    int chunkSize = 1000000; // one million bytes
    char *lotsOfMemory = (char *)malloc(chunkSize);
    if (lotsOfMemory == NULL) {
	cout << "ERROR: cannot allocate " << chunkSize << " bytes\n";
	exit(1);
    }
    MemUsage mu2;
    double memAfterAlloc_Mib = mu2.getMem();
    double reportedIncrease_Mib = memAfterAlloc_Mib - baseMem_Mib;
    double overhead_Mib = reportedIncrease_Mib - (chunkSize/(1024.*1024.));
    if (overhead_Mib < 0) {
	cout << "Base memory is " << baseMem_Mib << " Mebibytes\n";
	cout << "(see " << binaryUnitsURL <<" for Mebibytes)\n";
	cout << "Overhead is " << overhead_Mib << " Mebibytes\n";
	cout << "ERROR: reported memory decreased by " << 
				(-overhead_Mib) << " Mebibytes\n";
	exit(1);
    }

    //----------------------------------------------------------------
    // check if measured memory decreases when we free
    //----------------------------------------------------------------
    free(lotsOfMemory);
    MemUsage mu3;
    double memAfterFree_Mib = mu3.getMem();
    double reportedDecrease_Mib = memAfterAlloc_Mib - memAfterFree_Mib;
    if (reportedDecrease_Mib > 0) {
	cout << "Memory after malloc was " << 
				memAfterAlloc_Mib << " Mebibytes\n";
	cout << "Memory after free was   " << 
				memAfterFree_Mib << " Mebibytes\n";
	cout << "(see " << binaryUnitsURL <<" for Mebibytes)\n";
	cout << "Warning: reported memory decreased after free().  To get peak memory,\n";
	cout << "call getMem() after every allocation and save maximum.\n";
    }
}

int main(int argc, char *argv[])
{
    if (argc > 1 && strcmp(argv[1], "--version") == 0) {
	cout << "Selftest Version " << progVersion << endl;
	cout << versionRCS << endl;
//	cout << "using" << endl;
	exit(0);
    }

    if (argc != 1) {
	cout << "Wrong number of operands" << endl;
	cout << "use: " << useMsg << endl;
	exit(-1);
    }

    memoryTest();

    return 0;
}

// end of $Id$
