// $Id: denmat.h,v 1.8 2011/03/24 17:52:38 black Exp $
/***********************************
 * Author:        Matthew Cass     *
 * Email:         mcass@nist.gov   *
 * Date:          June 4, 2002     *
 *modified "Thu Mar 24 09:06:57 2011" *by "Paul E. Black"
 *                                 *
 * This is the header file for the *
 * density matrix class.           *
 **********************************/

// FILE: denmat.h
// CLASS PROVIDED: density matrix  
//
// CONSTRUCTOR:
//
// denmat(int numBits) allocate a density matrix for 
// the number of bits
// Precondition: size is a positive integer
// Postcondition: a density matrix of size 2^numBits by 2^numBits
//                is allocated.
//
// DESTRUCTOR:
//
// ~denmat()
// Postcondition: memory for the density matrix
//                has been deallocated
//
// CONSTANT MEMBER FUNCTIONS: 
//
// void print_state()
// Postcondition: the current state of the density
//                matrix is printed to standard out
//
// int bits()
// Postcondition: return the number of qubits the matrix represents
//
// int size()
// Postcondition: return the size of the matrix
// Note: size is 2^bits
//
// MODIFICATION MEMBER FUNCTIONS:
//
// void set_state(int row, int col, complex val)
// Precondition: 0 <= row and col < size and ||val|| <= 1.0 
// Postcondition: the state of the density matrix
//                at row row and column col is set
//                to val

#ifndef DENSITY_MATRIX
#define DENSITY_MATRIX

#include <complex.h>
#include <iostream>

// return the code version
std::string denmat$version(void);

class denmat
    {
    public:
    
        //CONSTRUCTOR & DESTRUCTOR
	denmat(int numBits);
        ~denmat();

        //CONSTANT MEMBER FUNCTIONS
        void print_state();
	float complex get_state(int row, int col){return matrix[row][col];}
	int bits(){return mat_bits;}
        int size(){return mat_size;}

	// check that density matrix is well-formed.
	// print a message if there is a problem.
	void check();

        //MODIFICATION MEMBER FUNCTIONS
        void set_state(int row, int col, float complex val);

	friend denmat *init_denmat(denmat *dp, float complex ket[]);

    private:

	//PRIVATE MEMBER VARIABLES
	int mat_bits;	// number of qubits
        int mat_size;	// should be 2^bits
	float complex **matrix;
    };

#endif
    
// end of $Source: /home/black/Quantum/QCSim/RCS/denmat.h,v $
