/**************************************
 * Author:        Andy Lane           *
 * Email:         alane@nist.gov      *
 * Date:          June 26, 2003       */
// *modified "Thu Mar 24 10:36:40 2011" *by "Paul E. Black"

//-----------------------------------------------------------------------------
//
// This software was developed at the National Institute of Standards 
// and Technology by employees of the Federal Government in the course 
// of their official duties.  Pursuant to title 17 Section 105 of the 
// United States Code this software is not subject to copyright 
// protection and is in the public domain. 
// 
// We would appreciate acknowledgment if the software is used.
//
// Paul E. Black  paul.black@nist.gov
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	Convert jaQuzzi (.jaq) files to QHDL
//-----------------------------------------------------------------------------

#include <iostream>
#include <stdlib.h>
#include <string.h>
#include "parsingFuncs.h"
#include "parseJAQ.h"

static std::string versionRCS =
"$Header: /home/black/Quantum/QCSim/RCS/jaq2qhdl.C,v 1.4 2011/03/25 13:59:37 black Exp $";

//-----------------------------------------------------------------------------
//	symbol table stuff
//-----------------------------------------------------------------------------

const int MAX_SYMBOLS = 50;	// max number of different symbols allowed

// information for one variable name
struct symbol_table_struct {
    char *name;		// name of symbol
    int qNo;		// qubit number, zero-based
};

symbol_table_struct symbol_table[MAX_SYMBOLS];

// number of variables in the symbol table
int symbolsInTable = 0;

void getSymbolByIndex(int ind, char* returnBuf)
{
    strcpy(returnBuf, symbol_table[ind].name);
}

// true if the symbol is added successfully
bool addToSymbolTab(char *name, int qubitNumb)
{
    // SKIMP: expand the table if we have no more room
    if (symbolsInTable >= MAX_SYMBOLS) return false;

    symbol_table[symbolsInTable].name = strdup(name);
    if (NULL == symbol_table[symbolsInTable].name) return false;
    symbol_table[symbolsInTable].qNo = qubitNumb;
    symbolsInTable++;

    return true;
}

// look up a name in the symbol table
int varNameLookup(char name[])
{
    for (int i = 0; i < symbolsInTable; i++) {
	if ((strcmp(name, symbol_table[i].name) == 0) &&
	    symbol_table[i].qNo != -1) {
	    return i;
	}
    }

    return -1;			// not found
}

static std::string useMsg = "qhdl2jaq [--version] file.qhdl";
static std::string qhdl2jaqVersion = "1.0";

int main(int argc, char *argv[])
{
    char *inputfile;
    int numQubits = 0;
    //    simstate *ssp;

    if (argc > 1 && strcmp(argv[1], "--version") == 0) {
	cout << "qhdl2jaq Version " << qhdl2jaqVersion << endl;
	cout << versionRCS << endl;
	cout << "using" << endl;
	cout << parseJAQ$version() << endl;
	cout << parsingFuncs$version() << endl;
	exit(0);
    }

    if (argc != 2) {
	cout << "Wrong number of operands" << endl;
	cout << "use: " << useMsg << endl;
	exit(-1);
    }

    inputfile = argv[1];

    ins.open(inputfile, ios::in);

    if (!ins) {
	cout << "Unable to open file: " << inputfile << endl;
	exit(-1);
    }

    // get the first symbol
    getNextSymbol();


    processDeclaredGates();

    numQubits = processQubitDecls();

    for(int i=0; i<numQubits; i++) {
	cout << "variable " << symbol_table[i].name << ":qubit;\n";
    }

    cout << "\n=|";
    for (int i=0; i<numQubits; i++) {
	cout << "0";
    }
    cout << ">;\n\n";

    processGates();

    ins.close();

    return 0;
}

// end of $Source: /home/black/Quantum/QCSim/RCS/jaq2qhdl.C,v $

