// *created  "Thu Aug 15 13:01:00 2002" *by "Paul E. Black"
// *modified "Thu Mar 24 10:33:42 2011" *by "Paul E. Black"

//-----------------------------------------------------------------------------
//
// This software was developed at the National Institute of Standards 
// and Technology by employees of the Federal Government in the course 
// of their official duties.  Pursuant to title 17 Section 105 of the 
// United States Code this software is not subject to copyright 
// protection and is in the public domain. 
// 
// We would appreciate acknowledgment if the software is used.
//
// Paul E. Black  paul.black@nist.gov
//
//-----------------------------------------------------------------------------

#include <math.h>
#include "operJaqOut.h"

static std::string versionRCS =
"$Header: /home/black/Quantum/QCSim/RCS/operJaqOut.C,v 1.11 2011/03/25 13:58:35 black Exp $";
std::string operations$version() {
    return versionRCS;
}

static int gateNo = 0; // counter for gates

static void print_gate(int numQubits, const char *gateName, int controlBit1, int controlBit2, int controlBit3, int controlBit4, int controlBit5, int controlBit6, int controlBit7, int controlBit8, int controlBit9, int targetBit)
{
    cout << "gate" << gateNo << "={";
    
    for (int i=0; i<numQubits; i++) {
	if (i==targetBit) {
	    cout << gateName;
	} else if (i==controlBit1 || i==controlBit2 || 
	          i==controlBit3 || i==controlBit4 || 
	          i==controlBit5 || i==controlBit6 || 
	          i==controlBit7 || i==controlBit8 ||
		   i==controlBit9) {
	    cout << "1";
	} else {
	    cout << "-";
	}
	if (i!=numQubits-1) {
	    cout << ":";
	}
    }
    cout << "}\n";

    gateNo++;
}

// compute the mask for CNOT
ParmErrorType cnot(int numQubits, int targetBit, int controlBit)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit < 0 || controlBit >= numQubits) {
	return QubitOOR;
    } else if (targetBit == controlBit) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

// compute the mask for Toffoli or CCNOT
ParmErrorType toffoli(int numQubits, int targetBit, int controlBit1, int controlBit2)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       controlBit1 == controlBit2) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}


// compute the mask for C3NOT
ParmErrorType c3not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || controlBit1 == controlBit2 ||
	       controlBit1 == controlBit3 || controlBit2 == controlBit3) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

// compute the mask for C4NOT
ParmErrorType c4not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3, int controlBit4)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ||
	controlBit4 < 0 || controlBit4 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || targetBit == controlBit4 ||
	       controlBit1 == controlBit2 || controlBit1 == controlBit3 || 
	       controlBit1 == controlBit4 || controlBit2 == controlBit3 ||
	       controlBit2 == controlBit4 || controlBit3 == controlBit4) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, controlBit4, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;

}

// compute the mask for C5NOT
ParmErrorType c5not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3, int controlBit4, int controlBit5)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ||
	controlBit4 < 0 || controlBit4 >= numQubits ||
	controlBit5 < 0 || controlBit5 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || targetBit == controlBit4 ||
	       targetBit == controlBit5 ||
	       controlBit1 == controlBit2 || controlBit1 == controlBit3 || 
	       controlBit1 == controlBit4 || controlBit1 == controlBit5 ||
	       controlBit2 == controlBit3 || controlBit2 == controlBit4 ||
	       controlBit2 == controlBit5 || controlBit3 == controlBit4 ||
	       controlBit3 == controlBit5 || controlBit4 == controlBit5) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, controlBit4, controlBit5, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

// compute the mask for C6NOT
ParmErrorType c6not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3, int controlBit4, int controlBit5, int controlBit6)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ||
	controlBit4 < 0 || controlBit4 >= numQubits ||
	controlBit5 < 0 || controlBit5 >= numQubits ||
	controlBit6 < 0 || controlBit6 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || targetBit == controlBit4 ||
	       targetBit == controlBit5 || targetBit == controlBit6 || 
	       controlBit1 == controlBit2 || controlBit1 == controlBit3 || 
	       controlBit1 == controlBit4 || controlBit1 == controlBit5 ||
	       controlBit1 == controlBit6 || controlBit2 == controlBit3 ||
	       controlBit2 == controlBit4 || controlBit2 == controlBit5 ||
	       controlBit2 == controlBit6 || controlBit3 == controlBit4 ||
	       controlBit3 == controlBit5 || controlBit3 == controlBit6 ||
	       controlBit4 == controlBit5 || controlBit4 == controlBit6 ||
	       controlBit5 == controlBit6) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, controlBit4, controlBit5, controlBit6, -1, -1, -1, targetBit);
    return NoParmError;
}

// compute the mask for C7NOT
ParmErrorType c7not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3, int controlBit4, int controlBit5, int controlBit6, int controlBit7)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ||
	controlBit4 < 0 || controlBit4 >= numQubits ||
	controlBit5 < 0 || controlBit5 >= numQubits ||
	controlBit6 < 0 || controlBit6 >= numQubits ||
	controlBit7 < 0 || controlBit7 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || targetBit == controlBit4 ||
	       targetBit == controlBit5 || targetBit == controlBit6 || 
	       targetBit == controlBit7 ||
	       controlBit1 == controlBit2 || controlBit1 == controlBit3 || 
	       controlBit1 == controlBit4 || controlBit1 == controlBit5 ||
	       controlBit1 == controlBit6 || controlBit1 == controlBit7 ||
	       controlBit2 == controlBit3 || controlBit2 == controlBit4 ||
	       controlBit2 == controlBit5 || controlBit2 == controlBit6 ||
	       controlBit2 == controlBit7 || controlBit3 == controlBit4 ||
	       controlBit3 == controlBit5 || controlBit3 == controlBit6 ||
	       controlBit3 == controlBit7 || controlBit4 == controlBit5 ||
	       controlBit4 == controlBit6 || controlBit4 == controlBit7 ||
	       controlBit5 == controlBit6 || controlBit5 == controlBit7 ||
	       controlBit6 == controlBit7) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, controlBit4, controlBit5, controlBit6, controlBit7, -1, -1, targetBit);
    return NoParmError;
}

// compute the mask for C8NOT
ParmErrorType c8not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3, int controlBit4, int controlBit5, int controlBit6, int controlBit7, int controlBit8)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ||
	controlBit4 < 0 || controlBit4 >= numQubits ||
	controlBit5 < 0 || controlBit5 >= numQubits ||
	controlBit6 < 0 || controlBit6 >= numQubits ||
	controlBit7 < 0 || controlBit7 >= numQubits ||
	controlBit8 < 0 || controlBit8 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || targetBit == controlBit4 ||
	       targetBit == controlBit5 || targetBit == controlBit6 || 
	       targetBit == controlBit7 || targetBit == controlBit8 ||
	       controlBit1 == controlBit2 || controlBit1 == controlBit3 || 
	       controlBit1 == controlBit4 || controlBit1 == controlBit5 ||
	       controlBit1 == controlBit6 || controlBit1 == controlBit7 ||
	       controlBit1 == controlBit8 || controlBit2 == controlBit3 ||
	       controlBit2 == controlBit4 || controlBit2 == controlBit5 ||
	       controlBit2 == controlBit6 || controlBit2 == controlBit7 ||
	       controlBit2 == controlBit8 || controlBit3 == controlBit4 ||
	       controlBit3 == controlBit5 || controlBit3 == controlBit6 ||
	       controlBit3 == controlBit7 || controlBit3 == controlBit8 ||
	       controlBit4 == controlBit5 || controlBit4 == controlBit6 ||
	       controlBit4 == controlBit7 || controlBit4 == controlBit8 ||
	       controlBit5 == controlBit6 || controlBit5 == controlBit7 ||
	       controlBit5 == controlBit8 || controlBit6 == controlBit7 ||
	       controlBit6 == controlBit8 || controlBit7 == controlBit8) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, controlBit4, controlBit5, controlBit6, controlBit7, controlBit8, -1, targetBit);
    return NoParmError;
}

// compute the mask for C9NOT
ParmErrorType c9not(int numQubits, int targetBit, int controlBit1, int controlBit2, int controlBit3, int controlBit4, int controlBit5, int controlBit6, int controlBit7, int controlBit8, int controlBit9)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit1 < 0 || controlBit1 >= numQubits ||
	controlBit2 < 0 || controlBit2 >= numQubits ||
	controlBit3 < 0 || controlBit3 >= numQubits ||
	controlBit4 < 0 || controlBit4 >= numQubits ||
	controlBit5 < 0 || controlBit5 >= numQubits ||
	controlBit6 < 0 || controlBit6 >= numQubits ||
	controlBit7 < 0 || controlBit7 >= numQubits ||
	controlBit8 < 0 || controlBit8 >= numQubits ||
	controlBit9 < 0 || controlBit9 >= numQubits ){
	return QubitOOR;
    } else if (targetBit == controlBit1 || targetBit == controlBit2 ||
	       targetBit == controlBit3 || targetBit == controlBit4 ||
	       targetBit == controlBit5 || targetBit == controlBit6 || 
	       targetBit == controlBit7 || targetBit == controlBit8 ||
	       targetBit == controlBit9 ||
	       controlBit1 == controlBit2 || controlBit1 == controlBit3 || 
	       controlBit1 == controlBit4 || controlBit1 == controlBit5 ||
	       controlBit1 == controlBit6 || controlBit1 == controlBit7 ||
	       controlBit1 == controlBit8 || controlBit1 == controlBit9 ||
	       controlBit2 == controlBit3 || controlBit2 == controlBit4 ||
	       controlBit2 == controlBit5 || controlBit2 == controlBit6 ||
	       controlBit2 == controlBit7 || controlBit2 == controlBit8 ||
	       controlBit2 == controlBit9 || controlBit3 == controlBit4 ||
	       controlBit3 == controlBit5 || controlBit3 == controlBit6 ||
	       controlBit3 == controlBit7 || controlBit3 == controlBit8 ||
	       controlBit3 == controlBit9 || controlBit4 == controlBit5 ||
	       controlBit4 == controlBit6 || controlBit4 == controlBit7 ||
	       controlBit4 == controlBit8 || controlBit4 == controlBit9 || 
	       controlBit5 == controlBit6 || controlBit5 == controlBit7 ||
	       controlBit5 == controlBit8 || controlBit5 == controlBit9 ||
	       controlBit6 == controlBit7 || controlBit6 == controlBit8 ||
	       controlBit6 == controlBit9 || controlBit7 == controlBit8 ||
	       controlBit7 == controlBit9 || controlBit8 == controlBit9) {
	return DupQubit;
    }

    print_gate(numQubits, "NOT", controlBit1, controlBit2, controlBit3, controlBit4, controlBit5, controlBit6, controlBit7, controlBit8, controlBit9, targetBit);
    return NoParmError;
}

// do controlled Z on targetBit in passed matrix
// this changes the passed matrix
ParmErrorType copZ(int numQubits, int targetBit, int controlBit)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit < 0 || controlBit >= numQubits) {
	return QubitOOR;
    } else if (targetBit == controlBit) {
	return DupQubit;
    }

    print_gate(numQubits, "sigma_z", controlBit, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

// do controlled hadamard on targetBit in passed matrix
// this changes the passed matrix
ParmErrorType chadamard(int numQubits, int targetBit, int controlBit)
{
    if (targetBit < 0 || targetBit >= numQubits ||
	controlBit < 0 || controlBit >= numQubits) {
	return QubitOOR;
    } else if (targetBit == controlBit) {
	return DupQubit;
    }

    print_gate(numQubits, "H", controlBit, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

// wrapper - simulation state is a density matrix
ParmErrorType hadamard(int numQubits, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits) {
	return QubitOOR;
    }

    print_gate(numQubits, "H", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

ParmErrorType opX(int numQubits, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits) {
	return QubitOOR;
    }

    print_gate(numQubits, "NOT", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}


// wrapper - simulation state is a density matrix
ParmErrorType opY(int numQubits, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits) {
	return QubitOOR;
    }

    print_gate(numQubits, "sigma_y", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

// wrapper - simulation state is a density matrix
ParmErrorType opZ(int numQubits, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits) {
	return QubitOOR;
    }

    print_gate(numQubits, "sigma_z", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}


// wrapper - simulation state is a density matrix
ParmErrorType probabilisticXYZ(int numQubits, float probP, float probQ, float probR, int targetBit)
{
    float probSum;
    probSum = probP + probQ + probR;

    if (targetBit < 0 || targetBit >= numQubits ){
	return QubitOOR;
    } else if (probP < 0.0 || probP > 1.0) {
	return ProbOOR;
    } else if (probQ < 0.0 || probQ > 1.0) {
	return ProbOOR;
    } else if (probR < 0.0 || probR > 1.0) {
	return ProbOOR;
    } else if (fabs(probSum - 1.0) < .00000000001) {
	// equal - do nothing
    } else if (float(probSum - 1.0) > float(0.0)) {
	return ProbOOR;
    }

    print_gate(numQubits, "", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1);
    return NoParmError;
}

// no correspondence in jaQuzzi.  Make a NO-OP
ParmErrorType genericOneQubit(int numQubits, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits ) {
	return QubitOOR;
    }

    print_gate(numQubits, "", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1);
    return NoParmError;
}

// wrapper - simulation state is a density matrix
ParmErrorType measure(int numQubits, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits ) {
	return QubitOOR;
    }

    print_gate(numQubits, "!", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    return NoParmError;
}

ParmErrorType genAmpDamp(int numQubits, float probDamp, float probZero, int targetBit)
{
    if (targetBit < 0 || targetBit >= numQubits ){
        return QubitOOR;
    } else if (probDamp < 0.0 || probDamp > 1.0) {
        return ProbOOR;
    } else if (probZero < 0.0 || probZero > 1.0) {
        return ProbOOR;
    }

    if (probDamp == 0.F) {
	// we have a NOOP
	print_gate(numQubits, "", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1);
    } else if (probDamp == 1.0F && probZero == 0.0F) { // reset to 1
	print_gate(numQubits, "d", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    } else if (probDamp == 1.0F && probZero == 1.0F) { // reset to 0
	print_gate(numQubits, "u", -1, -1, -1, -1, -1, -1, -1, -1, -1, targetBit);
    } else { // unrecognized, perform NOOP
	print_gate(numQubits, "", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1);
    }
	
    return NoParmError;
}


ParmErrorType BSWDepolCPhase(int numQubits, float probSuccess, int targetBit, int controlBit)
{
    if (targetBit < 0 || targetBit >= numQubits ){
	return QubitOOR;
    } else if (controlBit < 0 || controlBit >= numQubits ){
	return QubitOOR;
    } else if (probSuccess < 0.0 || probSuccess > 1.0) {
	return ProbOOR;
    }

    print_gate(numQubits, "", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1);
    return NoParmError;
}

// from CRC Standard Mathematical Tables, 14th ed., pg 16, 1965
static const float PI = 3.141592653589793238;

ParmErrorType rotateY(int numQubits, float angle, int qubit)
{
    if (angle < -PI || angle > 2*PI) {
	return ProbOOR;
    } else if (qubit < 0 || qubit >= numQubits) {
	return QubitOOR;
    }

    print_gate(numQubits, "", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1);
    return NoParmError;
}


// end of $Source: /home/black/Quantum/QCSim/RCS/operJaqOut.C,v $
