// $Id: operJaqOut.h,v 1.2 2011/03/25 13:58:44 black Exp $
// *created  "Thu Aug 15 13:01:00 2002" *by "Paul E. Black"
// *modified "Thu Mar 24 10:33:27 2011" *by "Paul E. Black"

//-----------------------------------------------------------------------------
//
// This software was developed at the National Institute of S
// and Technology by employees of the Federal Government in t
// of their official duties.  Pursuant to title 17 Section 10
// United States Code this software is not subject to copyrig
// protection and is in the public domain. 
// 
// We would appreciate acknowledgment if the software is used
//
// Paul E. Black  paul.black@nist.gov
//
//-----------------------------------------------------------------------------

#ifndef OPERATIONS_INCLUDED
#define OPERATIONS_INCLUDED

#include "parsingFuncs.h"

// return the code version
std::string operations$version(void);

// perform CNOT (flip targetBit if controlBit is 1)
ParmErrorType cnot(int numQubits, int targetBit, int controlBit);
// perform controlled Z
ParmErrorType copZ(int numQubits, int targetBit, int controlBit);
// perform Hadamard on targetBit
ParmErrorType hadamard(int numQubits, int targetBit);
// perform controlled Hadamard on targetBit if controlBit is 1
ParmErrorType chadamard(int numQubits, int targetBit, int controlBit);
// rotate angle radians about Y axis
ParmErrorType rotateY(int numQubits, float angle, int qubit);
// flip targetBit (Pauli X)
ParmErrorType opX(int numQubits, int targetBit);
// (Pauli Y)
ParmErrorType opY(int numQubits, int targetBit);
// (Pauli Z)
ParmErrorType opZ(int numQubits, int targetBit);
// perform Toffoli (flip targetBit if controlBits are 1)
ParmErrorType toffoli(int numQubits, int targetBit, int cntrlBit1, int cntrlBit2);
ParmErrorType c3not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3);
ParmErrorType c4not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3, int cBit4);
ParmErrorType c5not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3, int cBit4, int cBit5);
ParmErrorType c6not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3, int cBit4, int cBit5, int cBit6);
ParmErrorType c7not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3, int cBit4, int cBit5, int cBit6, int cBit7);
ParmErrorType c8not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3, int cBit4, int cBit5, int cBit6, int cBit7, int cBit8);
ParmErrorType c9not(int numQubits, int tBit, int cBit1, int cBit2, int cBit3, int cBit4, int cBit5, int cBit6, int cBit7, int cBit8, int cBit9);

//-----------------------------------------------------------------------------
//			 Error or Noisy Gates
//-----------------------------------------------------------------------------

// perform X, Y, or Z control targetBit with respective probabilities p, q, r
ParmErrorType probabilisticXYZ(int numQubits, float probP, float probQ, float probR, int targetBit);

// Generalized Amplitude Damping,  see Nielsen & Chuang Sect. 8.3.5 page 382
// gamma is the probability of damping
//	0 is no damping (I), 1 is force to p|0>+(1-p)|1>
// p is the probability of going to zero
ParmErrorType genAmpDamp(int numQubits, float gamma, float p, int targetBit);

// CPHASE specialization of depolarizing error 
// see Brennen, Song, and Williams  quant-ph/0301012 v2 23 Apr 2003
// p is the probability of no error
ParmErrorType BSWDepolCPhase(int numQubits, float p, int targetBit, int controlBit);

// perform noisy Hadamard on targetBit
// modeled after Eric Hsu's synthetic Hadamard
ParmErrorType noisyHadamard_01(int numQubits, int targetBit);

//-----------------------------------------------------------------------------
//			  Special Operations
//-----------------------------------------------------------------------------

ParmErrorType genericOneQubit(int numQubits, int targetBit);

// measure (reduce to classical states) targetBit
ParmErrorType measure(int numQubits, int targetBit);
// for dummy print gate command in operJaqOut.C
void print_state(int numQubits);
#endif

// end of $Source: /home/black/Quantum/QCSim/RCS/operJaqOut.h,v $
