// *created  "Fri Jul 25 11:17:02 2003" *by "Paul E. Black"
// *modified "Thu Mar 24 10:26:45 2011" *by "Paul E. Black"

//-----------------------------------------------------------------------------
//
// This software was developed at the National Institute of Standards 
// and Technology by employees of the Federal Government in the course 
// of their official duties.  Pursuant to title 17 Section 105 of the 
// United States Code this software is not subject to copyright 
// protection and is in the public domain. 
// 
// We would appreciate acknowledgment if the software is used.
//
// Paul E. Black  paul.black@nist.gov
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//	report resources (memory, time, etc.) used
//-----------------------------------------------------------------------------

#include <sys/time.h>
#include <sys/resource.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <iostream>

// GFV files
//#include "infolines.h"
//#include "timer.h"

static std::string versionRCS =
"$Header$";
std::string resource$version()
{
    return versionRCS;
}

// resources when this process started
static struct timeval start;
static int resourcesInitializedProperly = 0;

static double base_mem; // for memory usage
static double max_mem = 0; // the maximum memory usage

// capture resources (i.e., time of day) when process started
void initialResources(void)
{
    if (resourcesInitializedProperly) {
	printf("resources initialized again\n");
	return;
    }

    if (gettimeofday(&start, NULL)) {
	printf("gettimeofday failed: %d\n", errno);
	return;
    }

    // GFV
//    MemUsage mu;
//    base_mem = mu.getMem();

    resourcesInitializedProperly = 1;
}

// report resources used
void reportResources(void)
{
    struct rusage use;

    if (getrusage(RUSAGE_SELF, &use)) {
	printf("rusage failed: %d\n", errno);
	return;
    }

    // this reproduces the values and format of
    //	$ time qcsim ...
    // the real time is low (about .02s) probably because of load time, etc.

    struct timeval end;
    if (gettimeofday(&end, NULL)) {
	printf("real unavailable: gettimeofday failed: %d\n", errno);
    } else {
	int rmin = (end.tv_sec - start.tv_sec)/60;
	double rsec = (end.tv_sec - start.tv_sec) - (rmin*60) 
				+ (end.tv_usec - start.tv_usec)/1000000.;
	if (resourcesInitializedProperly) {
	    printf("real\t%dm%.3lfs (+.02s on keystone)\n", rmin, rsec);
	} else {
	    printf("real unavailable: resources not initialized properly\n");
	}
    }

    int umin = use.ru_utime.tv_sec/60;
    double usec = use.ru_utime.tv_sec - (umin*60) 
				+ use.ru_utime.tv_usec/1000000.;
    printf("user\t%dm%.3lfs\n", umin, usec);

    int smin = use.ru_stime.tv_sec/60;
    double ssec = use.ru_stime.tv_sec - (smin*60) 
				+ use.ru_stime.tv_usec/1000000.;
    printf("sys\t%dm%.3lfs\n", smin, ssec);

    // GFV
    printf("Memory Use (over base) (MiB): %f\n", max_mem - base_mem);
}

void saveMaxMemory()
{
#ifdef MEMORY_UTILITIES_CONVERTED
    MemUsage mu;
    double mem = mu.getMem();
    if (mem > max_mem) {
	max_mem = mem;
    }
#endif
}

// end of $Source$
